#!/bin/sh
#   Copyright (c) 2011 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of digitalSTROM Server.
#
#   netinfo.sh is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   netinfo.sh is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. ./errlib.sh
. ./jsonlib.sh
. ./util.sh

UPLOAD_SCRIPT="/www/scripts/uploadbackup.sh"
CRON_RANGE_HOURS=23

set_autoupload() {
    # first always try to remove from crontab to prevent duplicate entries
    TEMPFILE="`mktemp`"
    grep -v $UPLOAD_SCRIPT /etc/crontab > $TEMPFILE
    mv $TEMPFILE /etc/crontab

    if [[ "$1" == "on" ]]; then
        MAC=`/sbin/ifconfig eth0 | awk '{ print $5}' | sed "s/://g" | head -n 1`
        if [[ -z "$MAC" ]]; then
            log "Failed to retrieve mac address"
            return $ERR_SOFTWARE
        fi
        # lines below shamelessly stolen from Bryant's logfile uploader
        LOW_MAC=${MAC:6}
        LOW_MAC_DEC=$(printf "%d" "0x$LOW_MAC")
        if [[ ! "$LOW_MAC_DEC" ]] ; then
            LOW_MAC_DEC=0
        fi
        LOW_MAC_DEC_SCALED=$(expr $LOW_MAC_DEC \* 2)
        CRON_RANGE_MIN=$(expr $CRON_RANGE_HOURS \* 60)
        LOW_MAC_REMAINDER=$(expr $LOW_MAC_DEC_SCALED % $CRON_RANGE_MIN)

        # pseudo-randomly-select a time
        CRON_MIN=$(expr $LOW_MAC_REMAINDER % 60)
        CRON_HOUR=$(expr $LOW_MAC_REMAINDER / 60)
        TODAY=`date +"%u"`
        if [[ -z "$CRON_MIN" ]] || [[ -z "$CRON_HOUR" ]] || [[ -z "$TODAY" ]]; then
            log "Error, one of the variables is unset"
            log "CRON_MIN [$CRON_MIN], CRON_HOUR [$CRON_HOUR], TODAY [$TODAY]"
            return $ERR_SOFTWARE
        fi
        echo "$CRON_MIN $CRON_HOUR * * $TODAY root /www/scripts/uploadbackup.sh >/dev/null 2>&1" >> /etc/crontab

        # our MAC-randomized time may have passed today, but we did promise to
        # upload the backup on the day of activation
        NOW_MIN=`date +"%M"`
        NOW_HOUR=`date +"%H"`
        # lesser or equal because cron needs some time to reload
        if [[ $CRON_HOUR -le $NOW_HOUR ]] && [[ $CRON_MIN -le $NOW_MIN ]]; then
            log "Calculated random time already elapsed this day"
            RAND_MINUTES=$(awk 'BEGIN{srand();print int(rand()*59) }')
            if [[ -z "$RAND_MINUTES" ]]; then
                RAND_MINUTES=1
            fi
            log "Scheduling early upload in $RAND_MINUTES minute(s)"
            LOGMSG=$(echo "/bin/sh /www/scripts/uploadbackup.sh 0" | at now +$RAND_MINUTES minutes 2>&1)
            log "$LOGMSG"
        fi
    fi
    killall -HUP crond >/dev/null 2>&1
    return $ERR_OK
}

echo "Content-Type: application/json"
echo ""

# "$FORM_autouplad"


AUTO_UPLOAD=
AUTO_UPLOAD_RETSTATUS=

grep -q "$UPLOAD_SCRIPT" /etc/crontab > /dev/null 2>&1
RETCODE="$?"

if [[ "$RETCODE" -eq 0 ]]; then
    AUTO_UPLOAD="on"
    AUTO_UPLOAD_RETSTATUS="true"
elif [[ "$RETCODE" -eq 1 ]]; then
    AUTO_UPLOAD="off"
    AUTO_UPLOAD_RETSTATUS="false"
fi

if [[ "$1" == "status" ]]; then
    if [[ -z "$AUTO_UPLOAD_RETSTATUS" ]]; then
        json_error "$ERR_CONFIG" "Could not determine status of automatic upload configuration!"
        exit 1
    else
        echo -n "{ \"success\": true, \"code\": $ERR_OK, \"message\": \"OK\", \"data\": { \"autouploadbackups\": $AUTO_UPLOAD_RETSTATUS } }"
        exit 0
    fi
fi

if [[ "$1" == "on" ]] || [[ "$1" == "off" ]]; then
    if [[ "$1" != "$AUTO_UPLOAD" ]]; then
        set_autoupload "$1"
        RETCODE="$?"
        if [[ "$RETCODE" -ne "$ERR_OK" ]]; then
            json_error "$ERR_SOFTWARE" "Could not configure automatic backup upload"
            exit 1
        fi
    fi
else
    json_error $ERR_PARAM "Invalid parameter, must be \"on\", \"off\" or \"status\"!"
    exit 1
fi

json_ok
