#!/bin/sh
#   Copyright (c) 2013 digitalSTROM.org, Zurich, Switzerland
#
#   Author: Sergey 'Jin' Bostandzhyan <jin@mediatomb.cc>
#
#   This file is part of dSS11 WebSetup
#
#   This file is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This file is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this file. If not, see <http://www.gnu.org/licenses/>.

# source common functions
. /www/pages/cgi-bin/errlib.sh
. /www/pages/cgi-bin/jsonlib.sh
. /www/pages/cgi-bin/util.sh

PAGEKITE_CFG="/etc/default/tempkite"
THIS_SCRIPT="/www/pages/cgi-bin/tempkiteconfig.sh"

# $1 on/off
# $2 if "on" - deactivation timeout in hours
set_pagekite_service() {
    TEMPFILE="`mktemp`"
    grep -v $THIS_SCRIPT /etc/crontab > $TEMPFILE
    mv $TEMPFILE /etc/crontab >/dev/null 2>&1
    killall -HUP crond >/dev/null 2>&1

    sv down tempkite > /dev/null 2>&1
    rm -rf /etc/runit/tempkite
    if [[ "$1" == "on" ]]; then 
        mkdir -p /etc/runit/tempkite/log >/dev/null 2>&1
        cat > /etc/runit/tempkite/run << EOL
#!/bin/sh

CONFIG=$PAGEKITE_CFG
LOGDIR=/var/log/pagekite/
mkdir -p \$LOGDIR
chmod a+rx \$LOGDIR

if [[ ! -f \$CONFIG ]]; then
    sleep 10
    exit -1
fi

exec 2>&1
exec /usr/bin/pagekite-connector -c \$CONFIG
EOL
        chmod a+x /etc/runit/tempkite/run >/dev/null 2>&1
        cat > /etc/runit/tempkite/log/run << EOL
#!/bin/sh

LOGDIR=/var/log/pagekite
mkdir -p \$LOGDIR
chmod 750 \$LOGDIR

exec 2>&1
exec svlogd2 -m -n tempkite.log \$LOGDIR
EOL
        chmod a+x /etc/runit/tempkite/log/run >/dev/null 2>&1
        DATE_STRING=$(date --date='%s' -d "@$(( `date +%s`+$2*60*60 ))" +"%M %k %d %m")
        echo "$DATE_STRING * root $THIS_SCRIPT off >/dev/null 2>&1" >> /etc/crontab
        killall -HUP crond >/dev/null 2>&1
        sv up tempkite > /dev/null 2>&1
    elif [[ "$1" == "off" ]]; then
        if [[ -f $PAGEKITE_CFG ]]; then
            source $PAGEKITE_CFG
            if [[ -n "$DSS_TOKEN" ]]; then
                curl -s -H "Authorization: username=\"dssadmin\"" "http://127.0.0.1:8088/json/system/revokeToken?applicationToken=$DSS_TOKEN" 2>&1 >/dev/null
            fi
            grep -v "DSS_TOKEN" $PAGEKITE_CFG > $TEMPFILE
            mv $TEMPFILE $PAGEKITE_CFG >/dev/null 2>&1
        fi
    fi
}

# $1 name, $2 password
set_pagekite_config() {
cat > $PAGEKITE_CFG << EOL
NAME=$1
PROXYPW=$2
PROXYPORT=443
DOMAIN=.digitalstrom.net
DDNS=http://dnsup.digitalstrom.net/?hostname=%s&myip=%s&sign=%s
V4FRONTEND=fe.digitalstrom.net
V4PORT=443
DEBUG=0
NOSSL=0
EOL
}

if [[ "$1" != "on" ]] && [[ "$1" != "off" ]] && [[ "$1" != "configure" ]]; then
    log "Invalid remoteaccess parameter value!"
    exit 1
fi

if [[ "$1" != "configure" ]]; then
    if [[ "$1" == "on" ]] && [[ -z "$2" ]]; then
        log "Missing delay parameter!"
        exit 1
    fi

    log "Configuring temporary kite service with parameters $1 $2"
    set_pagekite_service "$1" "$2"
else
    if [[ -z "$2" ]] || [[ -z "$3" ]]; then
        log "Missing password or link parameter!"
        exit 1
    fi
    set_pagekite_config "$2"
fi

exit 0
